/*
 * Copyright (c) 2023, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 */

#ifndef VIRTNET_DPA_CQ_DIM_H
#define VIRTNET_DPA_CQ_DIM_H

#define VIRTNET_DPA_CQ_DIM_CQ_PERIOD_NUM_MODES 2
#define VIRTNET_DPA_CQ_DIM_RX_PROFILE_CNT 6
#define VIRTNET_DPA_CQ_DIM_RX_DEFAULT_PROFILE 2
#define VIRTNET_DPA_CQ_DIM_RX_FINDING_STABLE_TIMES 2
#define VIRTNET_DPA_CQ_DIM_DEFAULT_ON_TOP_TIRED 3
#define VIRTNET_DPA_CQ_DIM_PERCENTAGE 5

#define VIRTNET_DPA_CQ_DIM_RX_CQE_PROFILES {	\
	{0, 0},					\
	{16, 32},				\
	{32, 32},				\
	{64, 32},				\
	{256, 32},				\
	{1023, 32}				\
}

/*
 * enum virtnet_dpa_cq_dim_tune_state - CQ DIM algorithm tune states
 *
 * These will determine which action the algorithm should perform.
 *
 * @CQ_DIM_PARKING_ON_TOP: Algorithm found a local top point.
 *                      Exit on significant difference.
 * @CQ_DIM_PARKING_FINDING_STABLE: Algorithm is trying to find a stable traffic.
 *                              if pps is same for 3 period, will exit.
 * @CQ_DIM_GOING_RIGHT: Algorithm is currently trying higher moderation levels
 * @CQ_DIM_GOING_LEFT: Algorithm is currently trying lower moderation levels
 */
enum virtnet_dpa_cq_dim_tune_state {
	CQ_DIM_PARKING_FINDING_STABLE,
	CQ_DIM_PARKING_ON_TOP,
	CQ_DIM_GOING_RIGHT,
	CQ_DIM_GOING_LEFT,
};

struct virtnet_dpa_cq_dim_cq_moder {
	uint16_t usec;
	uint16_t pkts;
};

void *virtnet_dpa_cq_dim_func(void *context);
#endif
